-- every struct needs to have a unique name so rename this struct
struct s_prop_Template
(
	-- every modul needs this line and shoulden't be changed at all
	Base = rollout_AppDisplay.s_prop_Abstruct(),
	
	-- gradColors is an attribute that is used by the Base.generateGrads() function to color the rows
	gradColors = #(blue, green, yellow),
	
	-- this is an attribute that is used for the example and is not needed in every module
	numRandomGroups = 10,
	
	-- returns an array of strings representing the filters
	fn getFilters =
	(
		#("Odd", "Even")
	),
	
	-- return an array of indexes of rows that matches the given filter
	fn getRowsByFilter filt =
	(
		if filt as name == #odd then
			for i = 1 to numRandomGroups by 2 collect i
		else
			for i = 2 to numRandomGroups by 2 collect i
	),
	
	-- constructs the list
	fn init objs:objects forced:false =
	(
		if not Base.initialized or forced then (
			-- initiate attributes
			Base.Rows = #()						-- reset the row data
			Base.editLabels = true				-- users can edit labels (by double clicking on them and entering the new text)
			Base.canAdd = true					-- users can add a new row (by pressing the + button)
			Base.canTimeUpdate = true		-- users can set time update mode that causes the list to refresh when the current frame changes (by checking the clock check button)
			Base.materialsMode = true		-- users can set material mode to see a more detailed visualization of the data at render time (by checking the shaded shpere check button)
			
			if objs.count > 0 then (
				-- creating row structs for every row and setting the row label and value
				for i = 1 to numRandomGroups do (
					local val = Base.Rows.count + 1
					local NewRow = rollout_AppDisplay.s_rowData rowLabel:("Random group #" + val as string) val:val
					append Base.Rows NewRow
				)
				
				for o in objs do (
					-- append the current object into a random group
					local i = random 1 Base.Rows.count
					append Base.Rows[i].objs o
				)
				
				-- set the first row to the first color of the gradColors
				if Base.Rows.count > 0 then
					Base.Rows[1].keyColor = gradColors[1]
				
				-- set the last row to the last color of the gradColors
				if Base.Rows.count > 1 then
					Base.Rows[Base.Rows.count].keyColor = gradColors[3]
				
				-- set the mid row to the mid color of the gradColors
				if Base.Rows.count > 2 then
					Base.Rows[(Base.Rows.count * 0.5) as integer + 1].keyColor = gradColors[2]
				
				-- every row that doesn't have a color yet will receave a mix between the color of the closest row before 
				-- it that has a color and the closest row after it that has a color relative to the distabce from them.
				Base.generateGrads()
			)
			
			Base.initialized = true
		)
	),
	
	-- returns an array with the data needed to display the datagrid. There should be no reasone to overwrite this method.
	fn getRows =
	(
		Base.getRows()
	),
	
	-- this method is called every time the user presses one of the row colors.
	-- the row parameter holds the index of the pressed row (zero based).
	-- the objs parameter holds the selected objects at the time when the button was pressed.
	-- implement this method to change the properties in the object to match the properties represented by the row.
	-- after this method executes, the list will be automatically reconstructed.
	fn setValue row objs =
	(
		-- converting to 1 based array index
		row += 1
	),
	
	-- this method is called when the user pressed the add new row button
	-- implement it to add a new row to the Base.rows array
	-- after this method executes, the list will be automatically reconstructed.
	fn addRow =
	(
		numRandomGroups += 1
	),
	
	-- this method is called when the user edit the label of a group by doublclicking and typing.
	-- implemet this method to set the new label in a persistent way in the scene or else it will be gone next time the list is refreshed.
	fn setLabel row newLabel =
	(
		-- the default behaivior currently doesn't do anything
		Base.setLabel row newLabel
	),
	
	-- this method is used to create materials for objects according to their group membership.
	fn createMaterials =
	(
		-- the defalut behaivior will create a standard material using the Row's color
		Base.createMaterials()
	)
)

-- return an instance of the struct, don't forget to use the correct struct name here.
s_prop_Template()